
import socket
import sys
import select
import time
import os


# equivalent of 'socat - UNIX:/path/to/socket'
class TermSock(object):
    # docs for python 2.7 select reference select.PIPE_BUF,
    # which is at least 512 bytes (POSIX) and is the minimum
    # amount of bytes writable to a fd returned from select
    # as in the writable state.
    PIPE_BUF = 512

    STDIN_FD = sys.stdin.fileno()    # normally 0
    STDOUT_FD = sys.stdout.fileno()  # normally 1

    def __init__(self, sock_name):
        self.sock_name = sock_name
        self.stdin_log = []
        self.stdout_log = []
        self.sock_log = []
        self._out_fds = set()

    def _connect(self):
        # must hold on to socket object so it doesn't get GC'd
        # and closed, even though the only thing we care about
        # is its fileno()
        self.s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        while True:
            try:
                self.s.connect(self.sock_name)
            except socket.error:
                time.sleep(0.1)
            else:
                break

        self.s_fd = self.s.fileno()

    def _poll(self):
        """
        @return: whether we are finished.
        """
        r, w, e = select.select([TermSock.STDIN_FD,
                                 self.s_fd],
                                list(self._out_fds),
                                [])

        if e:
            return True
        if TermSock.STDIN_FD in r:
            stdin_input = os.read(TermSock.STDIN_FD,
                                  TermSock.PIPE_BUF).decode()
            if not stdin_input:
                return True
            self.stdin_log.append(stdin_input)
            self._out_fds.add(self.s_fd)
        if self.s_fd in r:
            sock_input = os.read(self.s_fd, TermSock.PIPE_BUF).decode()
            if not sock_input:
                return True
            self.sock_log.append(sock_input)
            self._out_fds.add(TermSock.STDOUT_FD)
        if TermSock.STDOUT_FD in w and self.sock_log:
            self.sock_log = ''.join(self.sock_log)
            wlen = os.write(TermSock.STDOUT_FD,
                            ''.join(self.sock_log).encode())
            remainder = self.sock_log[wlen:]
            if remainder:
                self.sock_log = [remainder]
            else:
                self.sock_log = []
                self._out_fds.remove(TermSock.STDOUT_FD)
        if self.s_fd in w and self.stdin_log:
            self.stdin_log = ''.join(self.stdin_log)
            wlen = os.write(self.s_fd, ''.join(self.stdin_log).encode())
            remainder = self.stdin_log[wlen:]
            if remainder:
                self.stdin_log = [remainder]
            else:
                self.stdin_log = []
                self._out_fds.remove(self.s_fd)

    def mainloop(self):
        self._connect()
        while not self._poll():
            pass


def terminal(addr):
    ts = TermSock(addr)
    ts.mainloop()


def main(args=None):
    if args is None:
        args = sys.argv[1:]
    if args:
        terminal(args[0])
    else:
        print("Usage: %s [debug socket address]" % sys.argv[0])

if __name__ == '__main__':
    main()



import Pyro.core
import updb

class PrimeServer(Pyro.core.ObjBase, updb.UPdb_mixin):
    def next_prime(self, p):
        """
        Naive, flaky and slow way to get the next
        prime number after that given.
        """
        if p < 2:
            # return first prime: 2
            return 2
        i = p  # start where we left off
        while True:
            i += 1
            probe = 2
            while True:
                if i % probe == 0:
                    # i is composite
                    break
                if probe >= p:
                    # i is co-prime to integers up to p.
                    return i
                probe += 1

def main():
    Pyro.core.initServer()
    daemon = Pyro.core.Daemon()
    daemon.connect(PrimeServer(), "rpc")
    daemon.requestLoop()

if __name__ == '__main__':
    main()



#!/usr/bin/python -u

# be compatible with Python 2.5
from __future__ import with_statement

try:
    from inspect import getcallargs, getargspec
except ImportError:
    from rpcpdb.inspect_helper import getcallargs, getargspec

import pdb
import socket
import sys
import os
import tempfile


class UPdb(pdb.Pdb):
    def __init__(self, sock_path, level=0, force=False):
        if force:
            try:
                os.remove(sock_path)
            except OSError:
                pass
        self._sock_path = sock_path
        self._level = level
        self._sock = socket.socket(socket.AF_UNIX,
                                   socket.SOCK_STREAM)
        self._sock.bind(self._sock_path)
        self._sock.listen(1)
        self._conn = self._sock.accept()[0]
        self._handle = self._conn.makefile('rw')
        self._old_stdin = sys.stdin
        self._old_stdout = sys.stdout
        extra = {}
        if 'nosigint' in getargspec(pdb.Pdb.__init__)[0]:
            # Python3.2 for some reason sets a SIGINT handler
            # when '(c)ontinue' is run (to allow resuming debugger
            # with Ctrl-C), but that fails when run in non-main thread,
            # so turn it off.
            extra = {'nosigint': True}
        pdb.Pdb.__init__(self,
                         stdin=self._handle,
                         stdout=self._handle,
                         **extra)
        sys.stdout = sys.stdin = self._handle

    def __enter__(self):
        target_frame = sys._getframe().f_back
        for count in range(self._level):
            try:
                target_frame = target_frame.f_back
            except AttributeError:
                break
        self.set_trace(target_frame)

    def __exit__(self, exc_type, exc_value, tb):
        sys.stdin = self._old_stdin
        sys.stdout = self._old_stdout

        self._conn.shutdown(socket.SHUT_RDWR)
        os.remove(self._sock_path)
        os.rmdir(os.path.dirname(self._sock_path))


class UPdb_mixin(object):
    """
    This is designed to be mixed-in to an RPC server
    object. It allows an RPC client to register a
    RPC method as being debuggable.

    On triggering, subsequent calls to that method
    will trigger a pdb breakpoint (currently with
    interaction over a unix socket)
    """
    # this is a map from function name to
    # (function, unix_socket_path) pairs.
    _updb_debug_func = {}

    def debug_func(self, f, once=True, force=True, match_criteria=None):
        """
        @param f: function name
        @param once: one-shot debug - undebug after first hit (default True)
        @param force: override any existing unix socket (default True)
        @param match_criteria: optional dictionary mapping parameter names
            to values; checked prior to debug
        @return: path to debug socket
        """
        if f in self._updb_debug_func:
            return self._updb_debug_func[f][1]
        else:
            func = getattr(self, f)
            pdb_sock_path = "%s/pdb_sock" % tempfile.mkdtemp(prefix='updb_')
            self._updb_debug_func[f] = (func, pdb_sock_path)

            def arg_match(o, k):
                # check whether function f is called with
                # parameters matching the given match_criteria
                if not match_criteria:
                    # we always match if we have no match criteria
                    return True
                called_with = getcallargs(func, *o, **k)
                # insert anything passed using **kwargs etc as if it were
                # a standard named parameter (for the purposes of comparison)
                fn_keyword_arg_name = getargspec(func)[2]
                if fn_keyword_arg_name is not None:
                    for kw_k, kw_v in called_with.get(fn_keyword_arg_name,
                                                      {}).items():
                        called_with[kw_k] = kw_v

                match = True
                # All things in match_criteria must have exact matches
                # in called_with for us to match things.
                for key, val in match_criteria.items():
                    # note short-circuit logic here
                    if key not in called_with or called_with[key] != val:
                        match = False

                return match

            def _(*o, **k):
                if arg_match(o, k):
                    if once:
                        self.undebug_func(f)
                    with UPdb(pdb_sock_path, level=1, force=force):
                        return func(*o, **k)
                else:
                    # we're not debugging you, this time...
                    return func(*o, **k)
            setattr(self, f, _)
            return pdb_sock_path

    def undebug_func(self, f):
        """
        don't debug this function anymore.
        """
        if f in self._updb_debug_func:
            setattr(self, f, self._updb_debug_func.pop(f)[0])
